unit InfoDialog;

interface

uses Windows, Messages, ShellAPI, NLib, NForm, Player;

type
  TInfoDlg = class(TModelessDlg)
    Icon: HICON;
    Player: TPlayer;
    CTime, ATime, MTime: TFileTime;
    procedure UpdateInfo;
    procedure WMDrawItem(var Message: TWMDrawItem); message WM_DRAWITEM;
    procedure WMInitDialog(var Message: TWMInitDialog); message WM_INITDIALOG;
    procedure WMCommand(var Message: TWMCommand); message WM_COMMAND; 
  end;

implementation

{$I dialogid.inc}

function GetIcon(const Path: string): HICON;
var
  id: Word;
begin
  id := 1; // if id=1, It is not thumbnail icon but normal file icon
  Result := ExtractAssociatedIcon(HInstance, PChar(Path), id);
end;

function BytesFormat(Value: Integer): string;
var
  s: string;
begin
  Result := UnitFormat(Value);
  s := Copy(Result, 1, Length(Result)-1);
  case Result[Length(Result)] of
    'G': Result := s + ' GB';
    'M': Result := s + ' MB';
    'K': Result := s + ' KB';
  else
    Result := Result + ' bytes';
  end;
end;

function NumFormat(Value: Integer): string;
var
  nf: TNumberFmt;
  buf: array[0..255] of Char;
begin
  nf.NumDigits := 0;
  nf.LeadingZero := 0;
  nf.Grouping := 3;
  nf.lpDecimalSep := '.';
  nf.lpThousandSep := ',';
  nf.NegativeOrder := 1;
  GetNumberFormat(0, 0, PChar(IStr(Value)), @nf, buf, SizeOf(buf));
  Result := buf;
end;

function DateTimeStr(const FileTime: TFileTime): string;
var
  ft: TFileTime;
  st: TSystemTime;
begin
  if (FileTime.dwLowDateTime = 0) and (FileTime.dwHighDateTime = 0) then
    Result := ''
  else begin
    FileTimeToLocalFileTime(FileTime, ft);
    FileTimeToSystemTime(ft, st);
    with st do
      Result := Fmt('%s-%02s-%02s %02s:%02s:%02s',
          [wYear, wMonth, wDay, wHour, wMinute, wSecond]);
  end;
end;

procedure TInfoDlg.UpdateInfo;
var
  fsize: Cardinal;
begin
  if Player.OpenedDvd then
  begin
    Icon := 0;
    fsize := 0;
    Items[IDC_FILENAME].SetText('<DVD>')
  end else
  begin
    Icon := GetIcon(Player.Path);
    with TFile.Create(Player.Path, 'r') do
      try
        fsize := size; 
      finally
        Free;
      end;
    Items[IDC_FILENAME].SetText(BaseName(Player.Path));
  end;

  Items[IDC_LOCATION].SetText(DirName(Player.Path));
  Items[IDC_LENGTH].SetText(Fmt('%02s:%02s:%02s', [Player.Len div 3600000,
    Player.Len mod 3600000 div 60000, Player.Len mod 60000 div 1000]));
  Items[IDC_FORMAT].SetText(Player.FormatInfo.Container);
 
  Items[IDC_SIZELABEL].Enable(Player.HasVideo);
  Items[IDC_FPSLABEL].Enable(Player.HasVideo);
  Items[IDC_FILEASPECTLABEL].Enable(Player.HasVideo);
  Items[IDC_VIDEOCODECLABEL].Enable(Player.HasVideo);
  if Player.HasVideo then
  begin
    Items[IDC_SIZE].SetText(Fmt('%sx%s', [Player.VideoWidth,
      Player.VideoHeight]));
    Items[IDC_FPS].SetText(Fmt('%s fps',
      [FStr(Player.FormatInfo.VideoFps, 3)]));
    Items[IDC_FILEASPECT].SetText(FStr(Player.VideoAspect, 4));
    Items[IDC_VIDEOCODEC].SetText(Player.FormatInfo.VideoCodec);
  end else
  begin
    Items[IDC_SIZE].SetText('');
    Items[IDC_FPS].SetText('');
    Items[IDC_FILEASPECT].SetText('');
    Items[IDC_VIDEOCODEC].SetText('');
  end;

  Items[IDC_AUDIOCHANNELS].SetText(IStr(Player.FormatInfo.AudioNumCh));
  Items[IDC_AUDIOSAMPLERATE].SetText(Fmt('%s Hz',
    [Player.FormatInfo.AudioSampleRate]));
  Items[IDC_AUDIOBITRATE].SetText(Fmt('%s bps',
    [Player.FormatInfo.AudioBitRate]));
  if Player.FormatInfo.AudioCodec = '' then
    Items[IDC_AUDIOCODEC].SetText('(unknown)')
  else
    Items[IDC_AUDIOCODEC].SetText(Player.FormatInfo.AudioCodec);

  Items[IDC_FILESIZE].SetText(Fmt('%s (%s bytes)', [BytesFormat(fsize),
    NumFormat(fsize)]));
  Items[IDC_CREATED].SetText(DateTimeStr(CTime));
  Items[IDC_MODIFIED].SetText(DateTimeStr(MTime));
  Items[IDC_ACCESSED].SetText(DateTimeStr(ATime));

  InvalidateRect(Items[IDC_ICON].Handle, nil, False);
end;

procedure TInfoDlg.WMDrawItem(var Message: TWMDrawItem);
begin
  with Message do
    if Ctl = IDC_ICON  then
      with DrawItemStruct^ do
      begin
        FillRect(hDC, rcItem, GetSysColorBrush(COLOR_3DFACE));
        DrawIcon(hDC, 0, 0, Icon);
        Result := 1;
      end;
end;

procedure TInfoDlg.WMInitDialog(var Message: TWMInitDialog);
begin
  UpdateInfo;
  Message.Result := 0;
end;

procedure TInfoDlg.WMCommand(var Message: TWMCommand);
begin
  with Message do
    case ItemID of
      IDOK, IDCANCEL:
        begin
          Result := 1;
          Close;
        end;
    end;
end;

end.

